module.exports = function(grunt) {

	grunt.loadNpmTasks('grunt-contrib-clean');
	grunt.loadNpmTasks('grunt-contrib-copy');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-ng-annotate');
	grunt.loadNpmTasks('grunt-import-js');

	var userConfig = require( './build.config.js' );

	var taskConfig = {
		pkg: grunt.file.readJSON("package.json"),
		meta: {
			banner:
			'/**\n' +
			' * <%= pkg.name %> - v<%= pkg.version %> - <%= grunt.template.today("yyyy-mm-dd") %>\n' +
			' * <%= pkg.homepage %>\n' +
			' *\n' +
			' * Copyright (c) <%= grunt.template.today("yyyy") %> <%= pkg.author %>\n' +
			' * Licensed <%= pkg.licenses.type %> <<%= pkg.licenses.url %>>\n' +
			' */\n'
		},
		import_js: {
			files: {
				expand: true,
				cwd: 'src/js/',
				src: ['**/Generated.js'],
				dest: '<%= temp_dir %>/',
				ext: '.js'
			}
		},
		clean: [
			'<%= compile_dir %>',
            '<%= build_dir %>'
		],
		concat: {
			compile_js: {
				options: {
					banner: '<%= meta.banner %>'
				},
				src: [
					'<%= vendor_files.js %>',
					'<%= compile_dir %>/<%= pkg.name %>.js'
				],
				dest: '<%= compile_dir %>/<%= pkg.name %>.min.js'
			},
			build_js: {
				options: {
					banner: '<%= meta.banner %>'
				},
				src: [
					'<%= temp_dir %>/**/*.js'
				],
				dest: '<%= compile_dir %>/<%= pkg.name %>.js'
			}
		},
		uglify: {
			compile: {
				options: {
					banner: '<%= meta.banner %>',
					mangle: false
				},
				files: {
					'<%= concat.compile_js.dest %>': '<%= concat.compile_js.dest %>'
				}
			}
		},
		karma: {
			options: {
				configFile: '<%= build_dir %>/karma-unit.js'
			},
			unit: {
				port: 9019,
				background: true
			},
			continuous: {
				singleRun: true
			}
		}
	};



	grunt.initConfig( grunt.util._.extend( userConfig, taskConfig ) );

	grunt.registerTask( 'compile', ['clean', 'import_js', 'concat:build_js', 'concat:compile_js', 'uglify']);
};