//OrderCloud JS SDK

(function (window) {

    var OrderCloud = {};

    function defineUrl(params, url) {
        for (var key in params) {
            url = url.indexOf('{' + key + '}') > -1 ? url.replace('{' + key + '}', params[key]) : url + (params[key] ? (url.indexOf('?') > -1 ? '&' + key + '=' + params[key] : '?' + key + '=' + params[key]) : '');
        }
        url = apiurl + '/' + url;
        return url;
    }

    
function setCookie(cname, cvalue) {
    document.cookie = cname + "=" + cvalue;
}

function getCookie(cname) {
    var name = cname + "=";
    var ca = document.cookie.split(';');
    for(var i=0; i<ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0)==' ') c = c.substring(1);
        if (c.indexOf(name) == 0) return c.substring(name.length,c.length);
    }
    return "";
}

function deleteCookie(cname) {
    document.cookie = cname + '=; expires=Thu, 01 Jan 1970 00:00:01 GMT;';
}

OrderCloud.Auth = {
    impersonating: false,
    appname: appname ? appname : 'OrderCloud',
    GetToken: function() {
        return this.impersonating ? getCookie(this.appname + 'impersonating.token').Token : getCookie(this.appname + '.token');
    },
    SetToken: function(token) {
        setCookie(this.appname + '.token', token);
    },
    RemoveToken: function() {
        deleteCookie(this.appname + '.token');
    },
    IsAuthenticated: function() {
        var deferred = Q.defer();

        if (getCookie(this.appname + '.token')) {
            deferred.resolve(getCookie(this.appname + '.token'));
        } else {
            deferred.reject();
        }
        return deferred.promise;
    },
    SetImpersonationToken: function(userID, token) {
        var tokenObject = {
            UserID: userID,
            Token: token
        };
        setCookie(this.appname + 'impersonating.token', tokenObject);
    },
    ClearImpersonationToken: function() {
        deleteCookie(this.appname + 'impersonating.token');
    },
    IsImpersonating: function() {
        return this.impersonating;
    },
    SetImpersonating: function(bool) {
        this.impersonating = bool;
    },
    GetImpersonating: function() {
        return getCookie(this.appname + 'impersonating.token');
    },
    Impersonate: function(token) {
        if (token) this.SetImpersonationToken(token);
        this.impersonating = true;
    }
};

    
OrderCloud.Credentials = {
    Get: function(credentials) {
        var deferred = Q.defer();
        var params = 'grant_type=password&scope=' + ocscope + '&client_id=' + clientid + '&username=' + credentials.Username + '&password=' + credentials.Password;
        var request = new XMLHttpRequest();
        request.onreadystatechange = function() {
            if (request.readyState != 4) return;
            if (request.status && request.status < 400) {
                deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
            }
            else {
                deferred.reject(request.responseText);
            }
        };
        request.open('POST', authurl, true);
        request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded' );
        request.send(params);
        return deferred.promise;
    },
    Delete: function() {
        OrderCloud.Auth.RemoveToken();
    }
};

    
OrderCloud.BuyerID = {
    Get: function() {
        var cookies = document.cookie;
        var name = "buyerID=";
        var ca = cookies.split(';');
        for (var i=0; i < ca.length; i++) {
            var c = ca[i];
            while(c.charAt(0)==' ') c = c.substring(1);
            if (c.indexOf(name) == 0) return c.substring(name.length, c.length);
        }
        return "";
    },
    Set: function(id) {
        document.cookie="buyerID=" + id;
    }
};

    OrderCloud.Addresses = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (addressID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'addressID': addressID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/{addressID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'address': address};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        },
        Update: function (addressID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'addressID': addressID, 'address': address};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/{addressID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        },
        Delete: function (addressID, overrideOrderConflict) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'addressID': addressID,
                'overrideOrderConflict': overrideOrderConflict
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/{addressID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (addressID, userID, userGroupID, level, isShipping, isBilling, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'addressID': addressID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'isShipping': isShipping,
                'isBilling': isBilling,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (addressID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'addressID': addressID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'assignment': assignment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(assignment);
            return deferred.promise;
        }
    };
    OrderCloud.ApiClients = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (clientID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'clientID': clientID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (api) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'api': api};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(api);
            return deferred.promise;
        },
        Update: function (clientID, client) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'clientID': clientID, 'client': client};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(client);
            return deferred.promise;
        },
        Delete: function (clientID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'clientID': clientID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.ApprovalRules = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (approvalRuleID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'approvalRuleID': approvalRuleID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules/{approvalRuleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (approvalRule) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'approvalRule': approvalRule};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(approvalRule);
            return deferred.promise;
        },
        Update: function (approvalRuleID, approvalRule) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'approvalRuleID': approvalRuleID,
                'approvalRule': approvalRule
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules/{approvalRuleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(approvalRule);
            return deferred.promise;
        },
        Delete: function (approvalRuleID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'approvalRuleID': approvalRuleID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules/{approvalRuleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Buyers = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function () {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (company) {
            var deferred = Q.defer();
            var params = {'company': company};
            var url = defineUrl(params, 'v1/buyers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(company);
            return deferred.promise;
        },
        Update: function (company) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'company': company};
            var url = defineUrl(params, 'v1/buyers/{buyerID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(company);
            return deferred.promise;
        }
    };
    OrderCloud.Categories = {
        List: function (search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'search': search,
                'depth': depth,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListChildren: function (parentID, search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'parentID': parentID,
                'search': search,
                'depth': depth,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{parentID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (categoryID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (category) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'category': category};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(category);
            return deferred.promise;
        },
        Update: function (categoryID, category) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID, 'category': category};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(category);
            return deferred.promise;
        },
        Delete: function (categoryID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Patch: function (categoryID, category) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID, 'category': category};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(category);
            return deferred.promise;
        },
        ListProductAssignments: function (categoryID, productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'categoryID': categoryID,
                'productID': productID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveProductAssignment: function (productAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'productAssignment': productAssignment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/productsassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(productAssignment);
            return deferred.promise;
        },
        DeleteProductAssignment: function (categoryID, productID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID, 'productID': productID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (categoryID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'categoryID': categoryID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (categoryID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'categoryID': categoryID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (categoryAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryAssignment': categoryAssignment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(categoryAssignment);
            return deferred.promise;
        }
    };
    OrderCloud.CostCenters = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (costCenterID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'costCenterID': costCenterID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/{costCenterID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (costCenter) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'costCenter': costCenter};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(costCenter);
            return deferred.promise;
        },
        Update: function (costCenterID, costCenter) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'costCenterID': costCenterID, 'costCenter': costCenter};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/{costCenterID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(costCenter);
            return deferred.promise;
        },
        Delete: function (costCenterID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'costCenterID': costCenterID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/{costCenterID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (costCenterID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'costCenterID': costCenterID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (costCenterID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'costCenterID': costCenterID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'assignment': assignment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(assignment);
            return deferred.promise;
        }
    };
    OrderCloud.Coupons = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (couponID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (coupon) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'coupon': coupon};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(coupon);
            return deferred.promise;
        },
        Update: function (couponID, coupon) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID, 'coupon': coupon};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(coupon);
            return deferred.promise;
        },
        Delete: function (couponID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignedProducts: function (couponID, search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/products');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteProductAssignment: function (couponID, productID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID, 'productID': productID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveProductAssignment: function (couponID, productID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID, 'productID': productID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignedCategories: function (couponID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        AssignCategory: function (couponID, categoryID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID, 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteCategoryAssignment: function (couponID, categoryID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID, 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (couponID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'assignment': assignment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(assignment);
            return deferred.promise;
        },
        DeleteAssignment: function (couponID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.CreditCards = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (creditCardID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'creditCardID': creditCardID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/{creditCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (card) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'card': card};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(card);
            return deferred.promise;
        },
        Update: function (creditCardID, card) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'creditCardID': creditCardID, 'card': card};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/{creditCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(card);
            return deferred.promise;
        },
        Delete: function (creditCardID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'creditCardID': creditCardID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/{creditCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (creditCardID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'creditCardID': creditCardID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'assignment': assignment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(assignment);
            return deferred.promise;
        },
        DeleteAssignment: function (creditCardID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'creditCardID': creditCardID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.EmailTemplates = {
        Get: function (emailTemplateType) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'emailTemplateType': emailTemplateType};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (emailTemplateType, emailTemplate) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'emailTemplateType': emailTemplateType,
                'emailTemplate': emailTemplate
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(emailTemplate);
            return deferred.promise;
        },
        Patch: function (emailTemplateType, emailTemplate) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'emailTemplateType': emailTemplateType,
                'emailTemplate': emailTemplate
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(emailTemplate);
            return deferred.promise;
        },
        ResetToDefault: function (emailTemplateType) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'emailTemplateType': emailTemplateType};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Files = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/files');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (fileID) {
            var deferred = Q.defer();
            var params = {'fileID': fileID};
            var url = defineUrl(params, 'v1/files/{fileID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        PostFileData: function (filename) {
            var deferred = Q.defer();
            var params = {'filename': filename};
            var url = defineUrl(params, 'v1/files');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.GiftCards = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/giftcards');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (giftCardID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'giftCardID': giftCardID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/giftcards/{giftCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (giftCard) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'giftCard': giftCard};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/giftcards');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(giftCard);
            return deferred.promise;
        },
        Update: function (giftCardID, giftCard) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'giftCardID': giftCardID, 'giftCard': giftCard};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/giftcards/{giftCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(giftCard);
            return deferred.promise;
        },
        Delete: function (giftCardID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'giftCardID': giftCardID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/giftcards/{giftCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Specs = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/specs');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (specID) {
            var deferred = Q.defer();
            var params = {'specID': specID};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Patch: function (specID, spec) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'spec': spec};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(spec);
            return deferred.promise;
        },
        Update: function (specID, spec) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'spec': spec};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(spec);
            return deferred.promise;
        },
        Create: function (spec) {
            var deferred = Q.defer();
            var params = {'spec': spec};
            var url = defineUrl(params, 'v1/specs');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(spec);
            return deferred.promise;
        },
        Delete: function (specID) {
            var deferred = Q.defer();
            var params = {'specID': specID};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListProductAssignments: function (specID, productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'productID': productID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/specs/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteProductAssignment: function (specID, productID) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'productID': productID};
            var url = defineUrl(params, 'v1/specs/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveProductAssignment: function (productAssignment) {
            var deferred = Q.defer();
            var params = {'productAssignment': productAssignment};
            var url = defineUrl(params, 'v1/specs/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(productAssignment);
            return deferred.promise;
        },
        CreateOption: function (specID, option) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'option': option};
            var url = defineUrl(params, 'v1/specs/{specID}/options');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(option);
            return deferred.promise;
        },
        UpdateOption: function (specID, optionID, option) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'optionID': optionID, 'option': option};
            var url = defineUrl(params, 'v1/specs/{specID}/options/{optionID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(option);
            return deferred.promise;
        },
        GetOption: function (specID, optionID) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'optionID': optionID};
            var url = defineUrl(params, 'v1/specs/{specID}/options/{optionID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteOption: function (specID, optionID) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'optionID': optionID};
            var url = defineUrl(params, 'v1/specs/{specID}/options/{optionID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.UserGroups = {
        List: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (userGroupID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (userGroupID, group) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID, 'group': group};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(group);
            return deferred.promise;
        },
        Patch: function (userGroupID, group) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID, 'group': group};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(group);
            return deferred.promise;
        },
        Create: function (group) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'group': group};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(group);
            return deferred.promise;
        },
        Delete: function (userGroupID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListMemberAssignments: function (userID, userGroupID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'userID': userID,
                'userGroupID': userGroupID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteMemberAssignment: function (userGroupID, userID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID, 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveMemberAssignment: function (userGroupAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupAssignment': userGroupAssignment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(userGroupAssignment);
            return deferred.promise;
        }
    };
    OrderCloud.LineItems = {
        Get: function (orderID, lineItemID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        List: function (orderID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (orderID, lineItem) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItem': lineItem};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(lineItem);
            return deferred.promise;
        },
        Update: function (orderID, lineItemID, lineItem) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'orderID': orderID,
                'lineItemID': lineItemID,
                'lineItem': lineItem
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(lineItem);
            return deferred.promise;
        },
        Patch: function (orderID, lineItemID, lineItem) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'orderID': orderID,
                'lineItemID': lineItemID,
                'lineItem': lineItem
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(lineItem);
            return deferred.promise;
        },
        Delete: function (orderID, lineItemID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SetShippingAddress: function (orderID, lineItemID, address) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'orderID': orderID,
                'lineItemID': lineItemID,
                'address': address
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        },
        PatchShippingAddress: function (orderID, lineItemID, address) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'orderID': orderID,
                'lineItemID': lineItemID,
                'address': address
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        }
    };
    OrderCloud.Me = {
        Get: function () {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/me');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListCostCenters: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/costcenters');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListUserGroups: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/me/usergroups');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAddresses: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/addresses');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListCategories: function (search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'depth': depth, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListSubcategories: function (parentID, search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {'parentID': parentID, 'search': search, 'depth': depth, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/categories/{parentID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListProducts: function (search, categoryID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'categoryID': categoryID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/products');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        GetProduct: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/me/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Orders = {
        Get: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        List: function (direction, from, to, search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'direction': direction,
                'buyerID': OrderCloud.BuyerID.Get(),
                'from': from,
                'to': to,
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/orders');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (order) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'order': order};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(order);
            return deferred.promise;
        },
        Update: function (orderID, order) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'order': order};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(order);
            return deferred.promise;
        },
        Patch: function (orderID, order) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'order': order};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(order);
            return deferred.promise;
        },
        Delete: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Submit: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/submit');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Approve: function (orderID, comments) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'comments': comments};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/approve');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Decline: function (orderID, comments) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'comments': comments};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/decline');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Cancel: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/cancel');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Ship: function (orderID, shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'shipment': shipment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/ship');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(shipment);
            return deferred.promise;
        },
        SetShippingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'address': address};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        },
        PatchShippingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'address': address};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        },
        SetBillingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'address': address};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/billto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        },
        PatchBillingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'address': address};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/billto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(address);
            return deferred.promise;
        }
    };
    OrderCloud.PasswordResets = {
        SendVerificationCode: function (passwordResetRequest) {
            var deferred = Q.defer();
            var params = {'passwordResetRequest': passwordResetRequest};
            var url = defineUrl(params, 'v1/password/reset');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(passwordResetRequest);
            return deferred.promise;
        },
        ResetPassword: function (verificationCode, passwordReset) {
            var deferred = Q.defer();
            var params = {'verificationCode': verificationCode, 'passwordReset': passwordReset};
            var url = defineUrl(params, 'v1/password/reset/{verificationCode}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(passwordReset);
            return deferred.promise;
        }
    };
    OrderCloud.PriceSchedules = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/priceschedules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (priceScheduleID) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (priceSchedule) {
            var deferred = Q.defer();
            var params = {'priceSchedule': priceSchedule};
            var url = defineUrl(params, 'v1/priceschedules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(priceSchedule);
            return deferred.promise;
        },
        Patch: function (priceScheduleID, priceSchedule) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID, 'priceSchedule': priceSchedule};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(priceSchedule);
            return deferred.promise;
        },
        Update: function (priceScheduleID, priceSchedule) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID, 'priceSchedule': priceSchedule};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(priceSchedule);
            return deferred.promise;
        },
        Delete: function (priceScheduleID) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SavePriceBreak: function (priceScheduleID, priceBreak) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID, 'priceBreak': priceBreak};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}/PriceBreaks');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(priceBreak);
            return deferred.promise;
        },
        DeletePriceBreak: function (priceScheduleID, quantity) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID, 'quantity': quantity};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}/PriceBreaks');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.AdminUsers = {
        Get: function (userID) {
            var deferred = Q.defer();
            var params = {'userID': userID};
            var url = defineUrl(params, 'v1/adminusers/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (userID, user) {
            var deferred = Q.defer();
            var params = {'userID': userID, 'user': user};
            var url = defineUrl(params, 'v1/adminusers/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(user);
            return deferred.promise;
        },
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/adminusers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Delete: function (userID) {
            var deferred = Q.defer();
            var params = {'userID': userID};
            var url = defineUrl(params, 'v1/adminusers/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (user) {
            var deferred = Q.defer();
            var params = {'user': user};
            var url = defineUrl(params, 'v1/adminusers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(user);
            return deferred.promise;
        }
    };
    OrderCloud.Shipments = {
        Get: function (shipmentID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        List: function (orderID, search, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'orderID': orderID,
                'search': search,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipment': shipment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(shipment);
            return deferred.promise;
        },
        Update: function (shipmentID, shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID, 'shipment': shipment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(shipment);
            return deferred.promise;
        },
        Patch: function (shipmentID, shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID, 'shipment': shipment};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(shipment);
            return deferred.promise;
        },
        Delete: function (shipmentID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveItem: function (shipmentID, item) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID, 'item': item};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}/items');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(item);
            return deferred.promise;
        },
        DeleteItem: function (shipmentID, orderID, lineItemID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'shipmentID': shipmentID,
                'orderID': orderID,
                'lineItemID': lineItemID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}/items');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.SpendingAccounts = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (spendingAccountID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'spendingAccountID': spendingAccountID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/{spendingAccountID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (spendingAccount) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'spendingAccount': spendingAccount};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(spendingAccount);
            return deferred.promise;
        },
        Update: function (spendingAccountID, spendingAccount) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'spendingAccountID': spendingAccountID,
                'spendingAccount': spendingAccount
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/{spendingAccountID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(spendingAccount);
            return deferred.promise;
        },
        Delete: function (spendingAccountID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'spendingAccountID': spendingAccountID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/{spendingAccountID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Products = {
        List: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/products');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (productID, product) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'product': product};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(product);
            return deferred.promise;
        },
        Patch: function (productID, product) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'product': product};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(product);
            return deferred.promise;
        },
        Create: function (product) {
            var deferred = Q.defer();
            var params = {'product': product};
            var url = defineUrl(params, 'v1/products');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(product);
            return deferred.promise;
        },
        Delete: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListVariants: function (productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/products/{productID}/variants');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListVariantInventory: function (productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/products/{productID}/variants/inventory');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        GetVariantInventory: function (productID, variantID) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID};
            var url = defineUrl(params, 'v1/products/{productID}/variants/inventory/{variantID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        UpdateVariantInventory: function (productID, variantID, inventory) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID, 'inventory': inventory};
            var url = defineUrl(params, 'v1/products/{productID}/variants/inventory/{variantID}/{inventory}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        UpdateVariant: function (productID, variantID, variant) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID, 'variant': variant};
            var url = defineUrl(params, 'v1/products/{productID}/variants/{variantID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(variant);
            return deferred.promise;
        },
        GetVariant: function (productID, variantID) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID};
            var url = defineUrl(params, 'v1/products/{productID}/variants/{variantID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        GetInventory: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}/inventory');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        UpdateInventory: function (productID, inventory) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'inventory': inventory};
            var url = defineUrl(params, 'v1/products/{productID}/inventory/{inventory}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (productAssignment) {
            var deferred = Q.defer();
            var params = {'productAssignment': productAssignment};
            var url = defineUrl(params, 'v1/products/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(productAssignment);
            return deferred.promise;
        },
        ListAssignments: function (productID, userID, userGroupID, level, priceScheduleID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'productID': productID,
                'buyerID': OrderCloud.BuyerID.Get(),
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'priceScheduleID': priceScheduleID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/products/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (productID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'productID': productID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/products/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Users = {
        List: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (userID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('GET', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (userID, user) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID, 'user': user};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(user);
            return deferred.promise;
        },
        Create: function (user) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'user': user};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(user);
            return deferred.promise;
        },
        Delete: function (userID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Patch: function (userID, user) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID, 'user': user};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(user);
            return deferred.promise;
        },
        GetAccessToken: function (userID, tokenRequest) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID, 'tokenRequest': tokenRequest};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}/accesstoken');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                if (request.status && request.status < 400) {
                    deferred.resolve(typeof(request.response) == 'string' ? JSON.parse(request.response) : request.response);
                }
                else {
                    deferred.reject();
                }
            };
            request.open('POST', url, true);
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send(tokenRequest);
            return deferred.promise;
        }
    };

    if (typeof exports === "object" && typeof module === "object") {
        module.exports = OrderCloud;
    } else {
        window.OrderCloud = OrderCloud;
    }
})(window, undefined);
